# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawShiftsClient
from ...requests.shift import ShiftParams
from ...core.request_options import RequestOptions
from ...types.create_shift_response import CreateShiftResponse
from ...requests.shift_query import ShiftQueryParams
from ...types.search_shifts_response import SearchShiftsResponse
from ...types.get_shift_response import GetShiftResponse
from ...types.update_shift_response import UpdateShiftResponse
from ...types.delete_shift_response import DeleteShiftResponse
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawShiftsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ShiftsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawShiftsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawShiftsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawShiftsClient
        """
        return self._raw_client

    def create(
        self,
        *,
        shift: ShiftParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateShiftResponse:
        """
        Creates a new `Shift`.

        A `Shift` represents a complete workday for a single team member.
        You must provide the following values in your request to this
        endpoint:

        - `location_id`
        - `team_member_id`
        - `start_at`

        An attempt to create a new `Shift` can result in a `BAD_REQUEST` error when:
        - The `status` of the new `Shift` is `OPEN` and the team member has another
        shift with an `OPEN` status.
        - The `start_at` date is in the future.
        - The `start_at` or `end_at` date overlaps another shift for the same team member.
        - The `Break` instances are set in the request and a break `start_at`
        is before the `Shift.start_at`, a break `end_at` is after
        the `Shift.end_at`, or both.

        Parameters
        ----------
        shift : ShiftParams
            The `Shift` to be created.

        idempotency_key : typing.Optional[str]
            A unique string value to ensure the idempotency of the operation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateShiftResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.labor.shifts.create(
            idempotency_key="HIDSNG5KS478L",
            shift={
                "location_id": "PAA1RJZZKXBFG",
                "start_at": "2019-01-25T03:11:00-05:00",
                "end_at": "2019-01-25T13:11:00-05:00",
                "wage": {
                    "title": "Barista",
                    "hourly_rate": {"amount": 1100, "currency": "USD"},
                    "tip_eligible": True,
                },
                "breaks": [
                    {
                        "start_at": "2019-01-25T06:11:00-05:00",
                        "end_at": "2019-01-25T06:16:00-05:00",
                        "break_type_id": "REGS1EQR1TPZ5",
                        "name": "Tea Break",
                        "expected_duration": "PT5M",
                        "is_paid": True,
                    }
                ],
                "team_member_id": "ormj0jJJZ5OZIzxrZYJI",
                "declared_cash_tip_money": {"amount": 500, "currency": "USD"},
            },
        )
        """
        response = self._raw_client.create(
            shift=shift, idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data

    def search(
        self,
        *,
        query: typing.Optional[ShiftQueryParams] = OMIT,
        limit: typing.Optional[int] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchShiftsResponse:
        """
        Returns a paginated list of `Shift` records for a business.
        The list to be returned can be filtered by:
        - Location IDs
        - Team member IDs
        - Shift status (`OPEN` or `CLOSED`)
        - Shift start
        - Shift end
        - Workday details

        The list can be sorted by:
        - `START_AT`
        - `END_AT`
        - `CREATED_AT`
        - `UPDATED_AT`

        Parameters
        ----------
        query : typing.Optional[ShiftQueryParams]
            Query filters.

        limit : typing.Optional[int]
            The number of resources in a page (200 by default).

        cursor : typing.Optional[str]
            An opaque cursor for fetching the next page.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchShiftsResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.labor.shifts.search(
            query={
                "filter": {
                    "workday": {
                        "date_range": {
                            "start_date": "2019-01-20",
                            "end_date": "2019-02-03",
                        },
                        "match_shifts_by": "START_AT",
                        "default_timezone": "America/Los_Angeles",
                    }
                }
            },
            limit=100,
        )
        """
        response = self._raw_client.search(query=query, limit=limit, cursor=cursor, request_options=request_options)
        return response.data

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetShiftResponse:
        """
        Returns a single `Shift` specified by `id`.

        Parameters
        ----------
        id : str
            The UUID for the `Shift` being retrieved.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetShiftResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.labor.shifts.get(
            id="id",
        )
        """
        response = self._raw_client.get(id, request_options=request_options)
        return response.data

    def update(
        self, id: str, *, shift: ShiftParams, request_options: typing.Optional[RequestOptions] = None
    ) -> UpdateShiftResponse:
        """
        Updates an existing `Shift`.

        When adding a `Break` to a `Shift`, any earlier `Break` instances in the `Shift` have
        the `end_at` property set to a valid RFC-3339 datetime string.

        When closing a `Shift`, all `Break` instances in the `Shift` must be complete with `end_at`
        set on each `Break`.

        Parameters
        ----------
        id : str
            The ID of the object being updated.

        shift : ShiftParams
            The updated `Shift` object.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateShiftResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.labor.shifts.update(
            id="id",
            shift={
                "location_id": "PAA1RJZZKXBFG",
                "start_at": "2019-01-25T03:11:00-05:00",
                "end_at": "2019-01-25T13:11:00-05:00",
                "wage": {
                    "title": "Bartender",
                    "hourly_rate": {"amount": 1500, "currency": "USD"},
                    "tip_eligible": True,
                },
                "breaks": [
                    {
                        "id": "X7GAQYVVRRG6P",
                        "start_at": "2019-01-25T06:11:00-05:00",
                        "end_at": "2019-01-25T06:16:00-05:00",
                        "break_type_id": "REGS1EQR1TPZ5",
                        "name": "Tea Break",
                        "expected_duration": "PT5M",
                        "is_paid": True,
                    }
                ],
                "version": 1,
                "team_member_id": "ormj0jJJZ5OZIzxrZYJI",
                "declared_cash_tip_money": {"amount": 500, "currency": "USD"},
            },
        )
        """
        response = self._raw_client.update(id, shift=shift, request_options=request_options)
        return response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> DeleteShiftResponse:
        """
        Deletes a `Shift`.

        Parameters
        ----------
        id : str
            The UUID for the `Shift` being deleted.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteShiftResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.labor.shifts.delete(
            id="id",
        )
        """
        response = self._raw_client.delete(id, request_options=request_options)
        return response.data


class AsyncShiftsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawShiftsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawShiftsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawShiftsClient
        """
        return self._raw_client

    async def create(
        self,
        *,
        shift: ShiftParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateShiftResponse:
        """
        Creates a new `Shift`.

        A `Shift` represents a complete workday for a single team member.
        You must provide the following values in your request to this
        endpoint:

        - `location_id`
        - `team_member_id`
        - `start_at`

        An attempt to create a new `Shift` can result in a `BAD_REQUEST` error when:
        - The `status` of the new `Shift` is `OPEN` and the team member has another
        shift with an `OPEN` status.
        - The `start_at` date is in the future.
        - The `start_at` or `end_at` date overlaps another shift for the same team member.
        - The `Break` instances are set in the request and a break `start_at`
        is before the `Shift.start_at`, a break `end_at` is after
        the `Shift.end_at`, or both.

        Parameters
        ----------
        shift : ShiftParams
            The `Shift` to be created.

        idempotency_key : typing.Optional[str]
            A unique string value to ensure the idempotency of the operation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateShiftResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.labor.shifts.create(
                idempotency_key="HIDSNG5KS478L",
                shift={
                    "location_id": "PAA1RJZZKXBFG",
                    "start_at": "2019-01-25T03:11:00-05:00",
                    "end_at": "2019-01-25T13:11:00-05:00",
                    "wage": {
                        "title": "Barista",
                        "hourly_rate": {"amount": 1100, "currency": "USD"},
                        "tip_eligible": True,
                    },
                    "breaks": [
                        {
                            "start_at": "2019-01-25T06:11:00-05:00",
                            "end_at": "2019-01-25T06:16:00-05:00",
                            "break_type_id": "REGS1EQR1TPZ5",
                            "name": "Tea Break",
                            "expected_duration": "PT5M",
                            "is_paid": True,
                        }
                    ],
                    "team_member_id": "ormj0jJJZ5OZIzxrZYJI",
                    "declared_cash_tip_money": {"amount": 500, "currency": "USD"},
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create(
            shift=shift, idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data

    async def search(
        self,
        *,
        query: typing.Optional[ShiftQueryParams] = OMIT,
        limit: typing.Optional[int] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchShiftsResponse:
        """
        Returns a paginated list of `Shift` records for a business.
        The list to be returned can be filtered by:
        - Location IDs
        - Team member IDs
        - Shift status (`OPEN` or `CLOSED`)
        - Shift start
        - Shift end
        - Workday details

        The list can be sorted by:
        - `START_AT`
        - `END_AT`
        - `CREATED_AT`
        - `UPDATED_AT`

        Parameters
        ----------
        query : typing.Optional[ShiftQueryParams]
            Query filters.

        limit : typing.Optional[int]
            The number of resources in a page (200 by default).

        cursor : typing.Optional[str]
            An opaque cursor for fetching the next page.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchShiftsResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.labor.shifts.search(
                query={
                    "filter": {
                        "workday": {
                            "date_range": {
                                "start_date": "2019-01-20",
                                "end_date": "2019-02-03",
                            },
                            "match_shifts_by": "START_AT",
                            "default_timezone": "America/Los_Angeles",
                        }
                    }
                },
                limit=100,
            )


        asyncio.run(main())
        """
        response = await self._raw_client.search(
            query=query, limit=limit, cursor=cursor, request_options=request_options
        )
        return response.data

    async def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetShiftResponse:
        """
        Returns a single `Shift` specified by `id`.

        Parameters
        ----------
        id : str
            The UUID for the `Shift` being retrieved.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetShiftResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.labor.shifts.get(
                id="id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(id, request_options=request_options)
        return response.data

    async def update(
        self, id: str, *, shift: ShiftParams, request_options: typing.Optional[RequestOptions] = None
    ) -> UpdateShiftResponse:
        """
        Updates an existing `Shift`.

        When adding a `Break` to a `Shift`, any earlier `Break` instances in the `Shift` have
        the `end_at` property set to a valid RFC-3339 datetime string.

        When closing a `Shift`, all `Break` instances in the `Shift` must be complete with `end_at`
        set on each `Break`.

        Parameters
        ----------
        id : str
            The ID of the object being updated.

        shift : ShiftParams
            The updated `Shift` object.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateShiftResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.labor.shifts.update(
                id="id",
                shift={
                    "location_id": "PAA1RJZZKXBFG",
                    "start_at": "2019-01-25T03:11:00-05:00",
                    "end_at": "2019-01-25T13:11:00-05:00",
                    "wage": {
                        "title": "Bartender",
                        "hourly_rate": {"amount": 1500, "currency": "USD"},
                        "tip_eligible": True,
                    },
                    "breaks": [
                        {
                            "id": "X7GAQYVVRRG6P",
                            "start_at": "2019-01-25T06:11:00-05:00",
                            "end_at": "2019-01-25T06:16:00-05:00",
                            "break_type_id": "REGS1EQR1TPZ5",
                            "name": "Tea Break",
                            "expected_duration": "PT5M",
                            "is_paid": True,
                        }
                    ],
                    "version": 1,
                    "team_member_id": "ormj0jJJZ5OZIzxrZYJI",
                    "declared_cash_tip_money": {"amount": 500, "currency": "USD"},
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.update(id, shift=shift, request_options=request_options)
        return response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> DeleteShiftResponse:
        """
        Deletes a `Shift`.

        Parameters
        ----------
        id : str
            The UUID for the `Shift` being deleted.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteShiftResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.labor.shifts.delete(
                id="id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.delete(id, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .tender import Tender
from .refund import Refund
from .transaction_product import TransactionProduct
from .address import Address
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Transaction(UncheckedBaseModel):
    """
    Represents a transaction processed with Square, either with the
    Connect API or with Square Point of Sale.

    The `tenders` field of this object lists all methods of payment used to pay in
    the transaction.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The transaction's unique ID, issued by Square payments servers.
    """

    location_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the transaction's associated location.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp for when the transaction was created, in RFC 3339 format.
    """

    tenders: typing.Optional[typing.List[Tender]] = pydantic.Field(default=None)
    """
    The tenders used to pay in the transaction.
    """

    refunds: typing.Optional[typing.List[Refund]] = pydantic.Field(default=None)
    """
    Refunds that have been applied to any tender in the transaction.
    """

    reference_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    If the transaction was created with the [Charge](api-endpoint:Transactions-Charge)
    endpoint, this value is the same as the value provided for the `reference_id`
    parameter in the request to that endpoint. Otherwise, it is not set.
    """

    product: typing.Optional[TransactionProduct] = pydantic.Field(default=None)
    """
    The Square product that processed the transaction.
    See [TransactionProduct](#type-transactionproduct) for possible values
    """

    client_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    If the transaction was created in the Square Point of Sale app, this value
    is the ID generated for the transaction by Square Point of Sale.
    
    This ID has no relationship to the transaction's canonical `id`, which is
    generated by Square's backend servers. This value is generated for bookkeeping
    purposes, in case the transaction cannot immediately be completed (for example,
    if the transaction is processed in offline mode).
    
    It is not currently possible with the Connect API to perform a transaction
    lookup by this value.
    """

    shipping_address: typing.Optional[Address] = pydantic.Field(default=None)
    """
    The shipping address provided in the request, if any.
    """

    order_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The order_id is an identifier for the order associated with this transaction, if any.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .money import MoneyParams
from ..types.loyalty_program_accrual_rule_tax_mode import LoyaltyProgramAccrualRuleTaxMode


class LoyaltyProgramAccrualRuleVisitDataParams(typing_extensions.TypedDict):
    """
    Represents additional data for rules with the `VISIT` accrual type.
    """

    minimum_amount_money: typing_extensions.NotRequired[MoneyParams]
    """
    The minimum purchase required during the visit to quality for points.
    """

    tax_mode: LoyaltyProgramAccrualRuleTaxMode
    """
    Indicates how taxes should be treated when calculating the purchase amount to determine whether the visit qualifies for points. 
    This setting applies only if `minimum_amount_money` is specified.
    See [LoyaltyProgramAccrualRuleTaxMode](#type-loyaltyprogramaccrualruletaxmode) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class DisputeEvidenceFileParams(typing_extensions.TypedDict):
    """
    A file to be uploaded as dispute evidence.
    """

    filename: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The file name including the file extension. For example: "receipt.tiff".
    """

    filetype: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Dispute evidence files must be application/pdf, image/heic, image/heif, image/jpeg, image/png, or image/tiff formats.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .timecard_wage import TimecardWage
from .break_ import Break
from .timecard_status import TimecardStatus
from .money import Money
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Timecard(UncheckedBaseModel):
    """
    A record of the hourly rate, start time, and end time of a single timecard (shift)
    for a team member. This might include a record of the start and end times of breaks
    taken during the shift.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    **Read only** The Square-issued UUID for this object.
    """

    location_id: str = pydantic.Field()
    """
    The ID of the [location](entity:Location) for this timecard. The location should be based on
    where the team member clocked in.
    """

    timezone: typing.Optional[str] = pydantic.Field(default=None)
    """
    **Read only** The time zone calculated from the location based on the `location_id`,
    provided as a convenience value. Format: the IANA time zone database identifier for the
    location time zone.
    """

    start_at: str = pydantic.Field()
    """
    The start time of the timecard, in RFC 3339 format and shifted to the location
    timezone + offset. Precision up to the minute is respected; seconds are truncated.
    """

    end_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The end time of the timecard, in RFC 3339 format and shifted to the location
    timezone + offset. Precision up to the minute is respected; seconds are truncated.
    """

    wage: typing.Optional[TimecardWage] = pydantic.Field(default=None)
    """
    Job and pay related information. If the wage is not set on create, it defaults to a wage
    of zero. If the title is not set on create, it defaults to the name of the role the team member
    is assigned to, if any.
    """

    breaks: typing.Optional[typing.List[Break]] = pydantic.Field(default=None)
    """
    A list of all the paid or unpaid breaks that were taken during this timecard.
    """

    status: typing.Optional[TimecardStatus] = pydantic.Field(default=None)
    """
    Describes the working state of the timecard.
    See [TimecardStatus](#type-timecardstatus) for possible values
    """

    version: typing.Optional[int] = pydantic.Field(default=None)
    """
    **Read only** The current version of the timecard, which is incremented with each update.
    This field is used for [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
    control to ensure that requests don't overwrite data from another request.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp of when the timecard was created, in RFC 3339 format presented as UTC.
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp of when the timecard was last updated, in RFC 3339 format presented as UTC.
    """

    team_member_id: str = pydantic.Field()
    """
    The ID of the [team member](entity:TeamMember) this timecard belongs to.
    """

    declared_cash_tip_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The cash tips declared by the team member for this timecard.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawRefundsClient
from ...requests.terminal_refund import TerminalRefundParams
from ...core.request_options import RequestOptions
from ...types.create_terminal_refund_response import CreateTerminalRefundResponse
from ...requests.terminal_refund_query import TerminalRefundQueryParams
from ...types.search_terminal_refunds_response import SearchTerminalRefundsResponse
from ...types.get_terminal_refund_response import GetTerminalRefundResponse
from ...types.cancel_terminal_refund_response import CancelTerminalRefundResponse
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawRefundsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RefundsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawRefundsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawRefundsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawRefundsClient
        """
        return self._raw_client

    def create(
        self,
        *,
        idempotency_key: str,
        refund: typing.Optional[TerminalRefundParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateTerminalRefundResponse:
        """
        Creates a request to refund an Interac payment completed on a Square Terminal. Refunds for Interac payments on a Square Terminal are supported only for Interac debit cards in Canada. Other refunds for Terminal payments should use the Refunds API. For more information, see [Refunds API](api:Refunds).

        Parameters
        ----------
        idempotency_key : str
            A unique string that identifies this `CreateRefund` request. Keys can be any valid string but
            must be unique for every `CreateRefund` request.

            See [Idempotency keys](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) for more information.

        refund : typing.Optional[TerminalRefundParams]
            The refund to create.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateTerminalRefundResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.terminal.refunds.create(
            idempotency_key="402a640b-b26f-401f-b406-46f839590c04",
            refund={
                "payment_id": "5O5OvgkcNUhl7JBuINflcjKqUzXZY",
                "amount_money": {"amount": 111, "currency": "CAD"},
                "reason": "Returning items",
                "device_id": "f72dfb8e-4d65-4e56-aade-ec3fb8d33291",
            },
        )
        """
        response = self._raw_client.create(
            idempotency_key=idempotency_key, refund=refund, request_options=request_options
        )
        return response.data

    def search(
        self,
        *,
        query: typing.Optional[TerminalRefundQueryParams] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        limit: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchTerminalRefundsResponse:
        """
        Retrieves a filtered list of Interac Terminal refund requests created by the seller making the request. Terminal refund requests are available for 30 days.

        Parameters
        ----------
        query : typing.Optional[TerminalRefundQueryParams]
            Queries the Terminal refunds based on given conditions and the sort order. Calling
            `SearchTerminalRefunds` without an explicit query parameter returns all available
            refunds with the default sort order.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for the original query.

        limit : typing.Optional[int]
            Limits the number of results returned for a single request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchTerminalRefundsResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.terminal.refunds.search(
            query={"filter": {"status": "COMPLETED"}},
            limit=1,
        )
        """
        response = self._raw_client.search(query=query, cursor=cursor, limit=limit, request_options=request_options)
        return response.data

    def get(
        self, terminal_refund_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetTerminalRefundResponse:
        """
        Retrieves an Interac Terminal refund object by ID. Terminal refund objects are available for 30 days.

        Parameters
        ----------
        terminal_refund_id : str
            The unique ID for the desired `TerminalRefund`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetTerminalRefundResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.terminal.refunds.get(
            terminal_refund_id="terminal_refund_id",
        )
        """
        response = self._raw_client.get(terminal_refund_id, request_options=request_options)
        return response.data

    def cancel(
        self, terminal_refund_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> CancelTerminalRefundResponse:
        """
        Cancels an Interac Terminal refund request by refund request ID if the status of the request permits it.

        Parameters
        ----------
        terminal_refund_id : str
            The unique ID for the desired `TerminalRefund`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CancelTerminalRefundResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.terminal.refunds.cancel(
            terminal_refund_id="terminal_refund_id",
        )
        """
        response = self._raw_client.cancel(terminal_refund_id, request_options=request_options)
        return response.data


class AsyncRefundsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawRefundsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawRefundsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawRefundsClient
        """
        return self._raw_client

    async def create(
        self,
        *,
        idempotency_key: str,
        refund: typing.Optional[TerminalRefundParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateTerminalRefundResponse:
        """
        Creates a request to refund an Interac payment completed on a Square Terminal. Refunds for Interac payments on a Square Terminal are supported only for Interac debit cards in Canada. Other refunds for Terminal payments should use the Refunds API. For more information, see [Refunds API](api:Refunds).

        Parameters
        ----------
        idempotency_key : str
            A unique string that identifies this `CreateRefund` request. Keys can be any valid string but
            must be unique for every `CreateRefund` request.

            See [Idempotency keys](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) for more information.

        refund : typing.Optional[TerminalRefundParams]
            The refund to create.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateTerminalRefundResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.terminal.refunds.create(
                idempotency_key="402a640b-b26f-401f-b406-46f839590c04",
                refund={
                    "payment_id": "5O5OvgkcNUhl7JBuINflcjKqUzXZY",
                    "amount_money": {"amount": 111, "currency": "CAD"},
                    "reason": "Returning items",
                    "device_id": "f72dfb8e-4d65-4e56-aade-ec3fb8d33291",
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create(
            idempotency_key=idempotency_key, refund=refund, request_options=request_options
        )
        return response.data

    async def search(
        self,
        *,
        query: typing.Optional[TerminalRefundQueryParams] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        limit: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchTerminalRefundsResponse:
        """
        Retrieves a filtered list of Interac Terminal refund requests created by the seller making the request. Terminal refund requests are available for 30 days.

        Parameters
        ----------
        query : typing.Optional[TerminalRefundQueryParams]
            Queries the Terminal refunds based on given conditions and the sort order. Calling
            `SearchTerminalRefunds` without an explicit query parameter returns all available
            refunds with the default sort order.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for the original query.

        limit : typing.Optional[int]
            Limits the number of results returned for a single request.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchTerminalRefundsResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.terminal.refunds.search(
                query={"filter": {"status": "COMPLETED"}},
                limit=1,
            )


        asyncio.run(main())
        """
        response = await self._raw_client.search(
            query=query, cursor=cursor, limit=limit, request_options=request_options
        )
        return response.data

    async def get(
        self, terminal_refund_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetTerminalRefundResponse:
        """
        Retrieves an Interac Terminal refund object by ID. Terminal refund objects are available for 30 days.

        Parameters
        ----------
        terminal_refund_id : str
            The unique ID for the desired `TerminalRefund`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetTerminalRefundResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.terminal.refunds.get(
                terminal_refund_id="terminal_refund_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(terminal_refund_id, request_options=request_options)
        return response.data

    async def cancel(
        self, terminal_refund_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> CancelTerminalRefundResponse:
        """
        Cancels an Interac Terminal refund request by refund request ID if the status of the request permits it.

        Parameters
        ----------
        terminal_refund_id : str
            The unique ID for the desired `TerminalRefund`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CancelTerminalRefundResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.terminal.refunds.cancel(
                terminal_refund_id="terminal_refund_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.cancel(terminal_refund_id, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class InvoiceRecipientTaxIds(UncheckedBaseModel):
    """
    Represents the tax IDs for an invoice recipient. The country of the seller account determines
    whether the corresponding `tax_ids` field is available for the customer. For more information,
    see [Invoice recipient tax IDs](https://developer.squareup.com/docs/invoices-api/overview#recipient-tax-ids).
    """

    eu_vat: typing.Optional[str] = pydantic.Field(default=None)
    """
    The EU VAT identification number for the invoice recipient. For example, `IE3426675K`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..types.country import Country
from ..types.currency import Currency
from ..types.bank_account_type import BankAccountType
import typing_extensions
import typing
from ..types.bank_account_status import BankAccountStatus


class BankAccountParams(typing_extensions.TypedDict):
    """
    Represents a bank account. For more information about
    linking a bank account to a Square account, see
    [Bank Accounts API](https://developer.squareup.com/docs/bank-accounts-api).
    """

    id: str
    """
    The unique, Square-issued identifier for the bank account.
    """

    account_number_suffix: str
    """
    The last few digits of the account number.
    """

    country: Country
    """
    The ISO 3166 Alpha-2 country code where the bank account is based.
    See [Country](#type-country) for possible values
    """

    currency: Currency
    """
    The 3-character ISO 4217 currency code indicating the operating
    currency of the bank account. For example, the currency code for US dollars
    is `USD`.
    See [Currency](#type-currency) for possible values
    """

    account_type: BankAccountType
    """
    The financial purpose of the associated bank account.
    See [BankAccountType](#type-bankaccounttype) for possible values
    """

    holder_name: str
    """
    Name of the account holder. This name must match the name 
    on the targeted bank account record.
    """

    primary_bank_identification_number: str
    """
    Primary identifier for the bank. For more information, see 
    [Bank Accounts API](https://developer.squareup.com/docs/bank-accounts-api).
    """

    secondary_bank_identification_number: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Secondary identifier for the bank. For more information, see 
    [Bank Accounts API](https://developer.squareup.com/docs/bank-accounts-api).
    """

    debit_mandate_reference_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Reference identifier that will be displayed to UK bank account owners
    when collecting direct debit authorization. Only required for UK bank accounts.
    """

    reference_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Client-provided identifier for linking the banking account to an entity
    in a third-party system (for example, a bank account number or a user identifier).
    """

    location_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The location to which the bank account belongs.
    """

    status: BankAccountStatus
    """
    Read-only. The current verification status of this BankAccount object.
    See [BankAccountStatus](#type-bankaccountstatus) for possible values
    """

    creditable: bool
    """
    Indicates whether it is possible for Square to send money to this bank account.
    """

    debitable: bool
    """
    Indicates whether it is possible for Square to take money from this 
    bank account.
    """

    fingerprint: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A Square-assigned, unique identifier for the bank account based on the
    account information. The account fingerprint can be used to compare account
    entries and determine if the they represent the same real-world bank account.
    """

    version: typing_extensions.NotRequired[int]
    """
    The current version of the `BankAccount`.
    """

    bank_name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Read only. Name of actual financial institution. 
    For example "Bank of America".
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CatalogCustomAttributeDefinitionSelectionConfigCustomAttributeSelection(UncheckedBaseModel):
    """
    A named selection for this `SELECTION`-type custom attribute definition.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique ID set by Square.
    """

    name: str = pydantic.Field()
    """
    Selection name, unique within `allowed_selections`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.custom_attribute_definition_visibility import CustomAttributeDefinitionVisibility
from .custom_attribute_definition import CustomAttributeDefinitionParams


class CustomAttributeParams(typing_extensions.TypedDict):
    """
    A custom attribute value. Each custom attribute value has a corresponding
    `CustomAttributeDefinition` object.
    """

    key: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The identifier
    of the custom attribute definition and its corresponding custom attributes. This value
    can be a simple key, which is the key that is provided when the custom attribute definition
    is created, or a qualified key, if the requesting
    application is not the definition owner. The qualified key consists of the application ID
    of the custom attribute definition owner
    followed by the simple key that was provided when the definition was created. It has the
    format application_id:simple key.
    
    The value for a simple key can contain up to 60 alphanumeric characters, periods (.),
    underscores (_), and hyphens (-).
    """

    value: typing_extensions.NotRequired[typing.Optional[typing.Any]]
    version: typing_extensions.NotRequired[int]
    """
    Read only. The current version of the custom attribute. This field is incremented when the custom attribute is changed.
    When updating an existing custom attribute value, you can provide this field
    and specify the current version of the custom attribute to enable
    [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency).
    This field can also be used to enforce strong consistency for reads. For more information about strong consistency for reads,
    see [Custom Attributes Overview](https://developer.squareup.com/docs/devtools/customattributes/overview).
    """

    visibility: typing_extensions.NotRequired[CustomAttributeDefinitionVisibility]
    """
    A copy of the `visibility` field value for the associated custom attribute definition.
    See [CustomAttributeDefinitionVisibility](#type-customattributedefinitionvisibility) for possible values
    """

    definition: typing_extensions.NotRequired[CustomAttributeDefinitionParams]
    """
    A copy of the associated custom attribute definition object. This field is only set when
    the optional field is specified on the request.
    """

    updated_at: typing_extensions.NotRequired[str]
    """
    The timestamp that indicates when the custom attribute was created or was most recently
    updated, in RFC 3339 format.
    """

    created_at: typing_extensions.NotRequired[str]
    """
    The timestamp that indicates when the custom attribute was created, in RFC 3339 format.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .catalog_info_response_limits import CatalogInfoResponseLimitsParams
from .standard_unit_description_group import StandardUnitDescriptionGroupParams


class CatalogInfoResponseParams(typing_extensions.TypedDict):
    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    limits: typing_extensions.NotRequired[CatalogInfoResponseLimitsParams]
    """
    Limits that apply to this API.
    """

    standard_unit_description_group: typing_extensions.NotRequired[StandardUnitDescriptionGroupParams]
    """
    Names and abbreviations for standard units.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class CheckoutMerchantSettingsPaymentMethodsPaymentMethodParams(typing_extensions.TypedDict):
    """
    The settings allowed for a payment method.
    """

    enabled: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether the payment method is enabled for the account.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .address import Address
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class FulfillmentRecipient(UncheckedBaseModel):
    """
    Information about the fulfillment recipient.
    """

    customer_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the customer associated with the fulfillment.
    
    If `customer_id` is provided, the fulfillment recipient's `display_name`,
    `email_address`, and `phone_number` are automatically populated from the
    targeted customer profile. If these fields are set in the request, the request
    values override the information from the customer profile. If the
    targeted customer profile does not contain the necessary information and
    these fields are left unset, the request results in an error.
    """

    display_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The display name of the fulfillment recipient. This field is required.
    
    If provided, the display name overrides the corresponding customer profile value
    indicated by `customer_id`.
    """

    email_address: typing.Optional[str] = pydantic.Field(default=None)
    """
    The email address of the fulfillment recipient.
    
    If provided, the email address overrides the corresponding customer profile value
    indicated by `customer_id`.
    """

    phone_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    The phone number of the fulfillment recipient. This field is required.
    
    If provided, the phone number overrides the corresponding customer profile value
    indicated by `customer_id`.
    """

    address: typing.Optional[Address] = pydantic.Field(default=None)
    """
    The address of the fulfillment recipient. This field is required.
    
    If provided, the address overrides the corresponding customer profile value
    indicated by `customer_id`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CustomerGroup(UncheckedBaseModel):
    """
    Represents a group of customer profiles.

    Customer groups can be created, be modified, and have their membership defined using
    the Customers API or within the Customer Directory in the Square Seller Dashboard or Point of Sale.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique Square-generated ID for the customer group.
    """

    name: str = pydantic.Field()
    """
    The name of the customer group.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the customer group was created, in RFC 3339 format.
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the customer group was last updated, in RFC 3339 format.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class EventTypeMetadata(UncheckedBaseModel):
    """
    Contains the metadata of a webhook event type.
    """

    event_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The event type.
    """

    api_version_introduced: typing.Optional[str] = pydantic.Field(default=None)
    """
    The API version at which the event type was introduced.
    """

    release_status: typing.Optional[str] = pydantic.Field(default=None)
    """
    The release status of the event type.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class MeasurementUnitCustomParams(typing_extensions.TypedDict):
    """
    The information needed to define a custom unit, provided by the seller.
    """

    name: str
    """
    The name of the custom unit, for example "bushel".
    """

    abbreviation: str
    """
    The abbreviation of the custom unit, such as "bsh" (bushel). This appears
    in the cart for the Point of Sale app, and in reports.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..types.error_category import ErrorCategory
from ..types.error_code import ErrorCode
import typing_extensions


class ErrorParams(typing_extensions.TypedDict):
    """
    Represents an error encountered during a request to the Connect API.

    See [Handling errors](https://developer.squareup.com/docs/build-basics/handling-errors) for more information.
    """

    category: ErrorCategory
    """
    The high-level category for the error.
    See [ErrorCategory](#type-errorcategory) for possible values
    """

    code: ErrorCode
    """
    The specific code of the error.
    See [ErrorCode](#type-errorcode) for possible values
    """

    detail: typing_extensions.NotRequired[str]
    """
    A human-readable description of the error for debugging purposes.
    """

    field: typing_extensions.NotRequired[str]
    """
    The name of the field provided in the original request (if any) that
    the error pertains to.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .money import Money
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class OrderLineItemModifier(UncheckedBaseModel):
    """
    A [CatalogModifier](entity:CatalogModifier).
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique ID that identifies the modifier only within this order.
    """

    catalog_object_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The catalog object ID referencing [CatalogModifier](entity:CatalogModifier).
    """

    catalog_version: typing.Optional[int] = pydantic.Field(default=None)
    """
    The version of the catalog object that this modifier references.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the item modifier.
    """

    quantity: typing.Optional[str] = pydantic.Field(default=None)
    """
    The quantity of the line item modifier. The modifier quantity can be 0 or more.
    For example, suppose a restaurant offers a cheeseburger on the menu. When a buyer orders
    this item, the restaurant records the purchase by creating an `Order` object with a line item
    for a burger. The line item includes a line item modifier: the name is cheese and the quantity
    is 1. The buyer has the option to order extra cheese (or no cheese). If the buyer chooses
    the extra cheese option, the modifier quantity increases to 2. If the buyer does not want
    any cheese, the modifier quantity is set to 0.
    """

    base_price_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The base price for the modifier.
    
    `base_price_money` is required for ad hoc modifiers.
    If both `catalog_object_id` and `base_price_money` are set, `base_price_money` will
    override the predefined [CatalogModifier](entity:CatalogModifier) price.
    """

    total_price_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The total price of the item modifier for its line item.
    This is the modifier's `base_price_money` multiplied by the line item's quantity.
    """

    metadata: typing.Optional[typing.Dict[str, typing.Optional[str]]] = pydantic.Field(default=None)
    """
    Application-defined data attached to this order. Metadata fields are intended
    to store descriptive references or associations with an entity in another system or store brief
    information about the object. Square does not process this field; it only stores and returns it
    in relevant API calls. Do not use metadata to store any sensitive information (such as personally
    identifiable information or card details).
    
    Keys written by applications must be 60 characters or less and must be in the character set
    `[a-zA-Z0-9_-]`. Entries can also include metadata generated by Square. These keys are prefixed
    with a namespace, separated from the key with a ':' character.
    
    Values have a maximum length of 255 characters.
    
    An application can have up to 10 entries per metadata field.
    
    Entries written by applications are private and can only be read or modified by the same
    application.
    
    For more information, see  [Metadata](https://developer.squareup.com/docs/build-basics/metadata).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class DateRangeParams(typing_extensions.TypedDict):
    """
    A range defined by two dates. Used for filtering a query for Connect v2
    objects that have date properties.
    """

    start_date: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A string in `YYYY-MM-DD` format, such as `2017-10-31`, per the ISO 8601
    extended format for calendar dates.
    The beginning of a date range (inclusive).
    """

    end_date: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A string in `YYYY-MM-DD` format, such as `2017-10-31`, per the ISO 8601
    extended format for calendar dates.
    The end of a date range (inclusive).
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SearchOrdersSourceFilter(UncheckedBaseModel):
    """
    A filter based on order `source` information.
    """

    source_names: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Filters by the [Source](entity:OrderSource) `name`. The filter returns any orders
    with a `source.name` that matches any of the listed source names.
    
    Max: 10 source names.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .delete_customer_response import DeleteCustomerResponse
import pydantic
from .error import Error
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class BulkDeleteCustomersResponse(UncheckedBaseModel):
    """
    Defines the fields included in the response body from the
    [BulkDeleteCustomers](api-endpoint:Customers-BulkDeleteCustomers) endpoint.
    """

    responses: typing.Optional[typing.Dict[str, DeleteCustomerResponse]] = pydantic.Field(default=None)
    """
    A map of responses that correspond to individual delete requests, represented by
    key-value pairs.
    
    Each key is the customer ID that was specified for a delete request and each value
    is the corresponding response.
    If the request succeeds, the value is an empty object (`{ }`).
    If the request fails, the value contains any errors that occurred during the request.
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Any top-level errors that prevented the bulk operation from running.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class FloatNumberRangeParams(typing_extensions.TypedDict):
    """
    Specifies a decimal number range.
    """

    start_at: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A decimal value indicating where the range starts.
    """

    end_at: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A decimal value indicating where the range ends.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .money import MoneyParams


class TenderCashDetailsParams(typing_extensions.TypedDict):
    """
    Represents the details of a tender with `type` `CASH`.
    """

    buyer_tendered_money: typing_extensions.NotRequired[MoneyParams]
    """
    The total amount of cash provided by the buyer, before change is given.
    """

    change_back_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of change returned to the buyer.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .tip_settings import TipSettingsParams


class DeviceCheckoutOptionsParams(typing_extensions.TypedDict):
    device_id: str
    """
    The unique ID of the device intended for this `TerminalCheckout`.
    A list of `DeviceCode` objects can be retrieved from the /v2/devices/codes endpoint.
    Match a `DeviceCode.device_id` value with `device_id` to get the associated device code.
    """

    skip_receipt_screen: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Instructs the device to skip the receipt screen. Defaults to false.
    """

    collect_signature: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates that signature collection is desired during checkout. Defaults to false.
    """

    tip_settings: typing_extensions.NotRequired[TipSettingsParams]
    """
    Tip-specific settings.
    """

    show_itemized_cart: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Show the itemization screen prior to taking a payment. This field is only meaningful when the
    checkout includes an order ID. Defaults to true.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing
from .bulk_delete_location_custom_attributes_response_location_custom_attribute_delete_response import (
    BulkDeleteLocationCustomAttributesResponseLocationCustomAttributeDeleteResponseParams,
)
import typing_extensions
from .error import ErrorParams


class BulkDeleteLocationCustomAttributesResponseParams(typing_extensions.TypedDict):
    """
    Represents a [BulkDeleteLocationCustomAttributes](api-endpoint:LocationCustomAttributes-BulkDeleteLocationCustomAttributes) response,
    which contains a map of responses that each corresponds to an individual delete request.
    """

    values: typing.Dict[str, BulkDeleteLocationCustomAttributesResponseLocationCustomAttributeDeleteResponseParams]
    """
    A map of responses that correspond to individual delete requests. Each response has the
    same key as the corresponding request.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .bank_account import BankAccountParams


class ListBankAccountsResponseParams(typing_extensions.TypedDict):
    """
    Response object returned by ListBankAccounts.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Information on errors encountered during the request.
    """

    bank_accounts: typing_extensions.NotRequired[typing.Sequence[BankAccountParams]]
    """
    List of BankAccounts associated with this account.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    When a response is truncated, it includes a cursor that you can 
    use in a subsequent request to fetch next set of bank accounts.
    If empty, this is the final response.
    
    For more information, see [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination).
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class DisputedPaymentParams(typing_extensions.TypedDict):
    """
    The payment the cardholder disputed.
    """

    payment_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Square-generated unique ID of the payment being disputed.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.inventory_change_type import InventoryChangeType
from ..types.inventory_state import InventoryState


class BatchRetrieveInventoryChangesRequestParams(typing_extensions.TypedDict):
    catalog_object_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    The filter to return results by `CatalogObject` ID.
    The filter is only applicable when set. The default value is null.
    """

    location_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    The filter to return results by `Location` ID.
    The filter is only applicable when set. The default value is null.
    """

    types: typing_extensions.NotRequired[typing.Optional[typing.Sequence[InventoryChangeType]]]
    """
    The filter to return results by `InventoryChangeType` values other than `TRANSFER`.
    The default value is `[PHYSICAL_COUNT, ADJUSTMENT]`.
    """

    states: typing_extensions.NotRequired[typing.Optional[typing.Sequence[InventoryState]]]
    """
    The filter to return `ADJUSTMENT` query results by
    `InventoryState`. This filter is only applied when set.
    The default value is null.
    """

    updated_after: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The filter to return results with their `calculated_at` value
    after the given time as specified in an RFC 3339 timestamp.
    The default value is the UNIX epoch of (`1970-01-01T00:00:00Z`).
    """

    updated_before: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The filter to return results with their `created_at` or `calculated_at` value
    strictly before the given time as specified in an RFC 3339 timestamp.
    The default value is the UNIX epoch of (`1970-01-01T00:00:00Z`).
    """

    cursor: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A pagination cursor returned by a previous call to this endpoint.
    Provide this to retrieve the next set of results for the original query.
    
    See the [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination) guide for more information.
    """

    limit: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The number of [records](entity:InventoryChange) to return.
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..requests.money import MoneyParams
from ..requests.address import AddressParams
from ..requests.cash_payment_details import CashPaymentDetailsParams
from ..requests.external_payment_details import ExternalPaymentDetailsParams
from ..requests.customer_details import CustomerDetailsParams
from ..requests.offline_payment_details import OfflinePaymentDetailsParams
from ..core.request_options import RequestOptions
from ..core.http_response import HttpResponse
from ..types.create_payment_response import CreatePaymentResponse
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.cancel_payment_by_idempotency_key_response import CancelPaymentByIdempotencyKeyResponse
from ..types.get_payment_response import GetPaymentResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..requests.payment import PaymentParams
from ..types.update_payment_response import UpdatePaymentResponse
from ..types.cancel_payment_response import CancelPaymentResponse
from ..types.complete_payment_response import CompletePaymentResponse
from ..core.client_wrapper import AsyncClientWrapper
from ..core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawPaymentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        source_id: str,
        idempotency_key: str,
        amount_money: typing.Optional[MoneyParams] = OMIT,
        tip_money: typing.Optional[MoneyParams] = OMIT,
        app_fee_money: typing.Optional[MoneyParams] = OMIT,
        delay_duration: typing.Optional[str] = OMIT,
        delay_action: typing.Optional[str] = OMIT,
        autocomplete: typing.Optional[bool] = OMIT,
        order_id: typing.Optional[str] = OMIT,
        customer_id: typing.Optional[str] = OMIT,
        location_id: typing.Optional[str] = OMIT,
        team_member_id: typing.Optional[str] = OMIT,
        reference_id: typing.Optional[str] = OMIT,
        verification_token: typing.Optional[str] = OMIT,
        accept_partial_authorization: typing.Optional[bool] = OMIT,
        buyer_email_address: typing.Optional[str] = OMIT,
        buyer_phone_number: typing.Optional[str] = OMIT,
        billing_address: typing.Optional[AddressParams] = OMIT,
        shipping_address: typing.Optional[AddressParams] = OMIT,
        note: typing.Optional[str] = OMIT,
        statement_description_identifier: typing.Optional[str] = OMIT,
        cash_details: typing.Optional[CashPaymentDetailsParams] = OMIT,
        external_details: typing.Optional[ExternalPaymentDetailsParams] = OMIT,
        customer_details: typing.Optional[CustomerDetailsParams] = OMIT,
        offline_payment_details: typing.Optional[OfflinePaymentDetailsParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreatePaymentResponse]:
        """
        Creates a payment using the provided source. You can use this endpoint
        to charge a card (credit/debit card or
        Square gift card) or record a payment that the seller received outside of Square
        (cash payment from a buyer or a payment that an external entity
        processed on behalf of the seller).

        The endpoint creates a
        `Payment` object and returns it in the response.

        Parameters
        ----------
        source_id : str
            The ID for the source of funds for this payment.
            This could be a payment token generated by the Web Payments SDK for any of its
            [supported methods](https://developer.squareup.com/docs/web-payments/overview#explore-payment-methods),
            including cards, bank transfers, Afterpay or Cash App Pay. If recording a payment
            that the seller received outside of Square, specify either "CASH" or "EXTERNAL".
            For more information, see
            [Take Payments](https://developer.squareup.com/docs/payments-api/take-payments).

        idempotency_key : str
            A unique string that identifies this `CreatePayment` request. Keys can be any valid string
            but must be unique for every `CreatePayment` request.

            Note: The number of allowed characters might be less than the stated maximum, if multi-byte
            characters are used.

            For more information, see [Idempotency](https://developer.squareup.com/docs/working-with-apis/idempotency).

        amount_money : typing.Optional[MoneyParams]
            The amount of money to accept for this payment, not including `tip_money`.

            The amount must be specified in the smallest denomination of the applicable currency
            (for example, US dollar amounts are specified in cents). For more information, see
            [Working with Monetary Amounts](https://developer.squareup.com/docs/build-basics/working-with-monetary-amounts).

            The currency code must match the currency associated with the business
            that is accepting the payment.

        tip_money : typing.Optional[MoneyParams]
            The amount designated as a tip, in addition to `amount_money`.

            The amount must be specified in the smallest denomination of the applicable currency
            (for example, US dollar amounts are specified in cents). For more information, see
            [Working with Monetary Amounts](https://developer.squareup.com/docs/build-basics/working-with-monetary-amounts).

            The currency code must match the currency associated with the business
            that is accepting the payment.

        app_fee_money : typing.Optional[MoneyParams]
            The amount of money that the developer is taking as a fee
            for facilitating the payment on behalf of the seller.

            The amount cannot be more than 90% of the total amount of the payment.

            The amount must be specified in the smallest denomination of the applicable currency
            (for example, US dollar amounts are specified in cents). For more information, see
            [Working with Monetary Amounts](https://developer.squareup.com/docs/build-basics/working-with-monetary-amounts).

            The fee currency code must match the currency associated with the seller
            that is accepting the payment. The application must be from a developer
            account in the same country and using the same currency code as the seller.

            For more information about the application fee scenario, see
            [Take Payments and Collect Fees](https://developer.squareup.com/docs/payments-api/take-payments-and-collect-fees).

            To set this field, `PAYMENTS_WRITE_ADDITIONAL_RECIPIENTS` OAuth permission is required.
            For more information, see [Permissions](https://developer.squareup.com/docs/payments-api/take-payments-and-collect-fees#permissions).

        delay_duration : typing.Optional[str]
            The duration of time after the payment's creation when Square automatically
            either completes or cancels the payment depending on the `delay_action` field value.
            For more information, see
            [Time threshold](https://developer.squareup.com/docs/payments-api/take-payments/card-payments/delayed-capture#time-threshold).

            This parameter should be specified as a time duration, in RFC 3339 format.

            Note: This feature is only supported for card payments. This parameter can only be set for a delayed
            capture payment (`autocomplete=false`).

            Default:

            - Card-present payments: "PT36H" (36 hours) from the creation time.
            - Card-not-present payments: "P7D" (7 days) from the creation time.

        delay_action : typing.Optional[str]
            The action to be applied to the payment when the `delay_duration` has elapsed. The action must be
            CANCEL or COMPLETE. For more information, see
            [Time Threshold](https://developer.squareup.com/docs/payments-api/take-payments/card-payments/delayed-capture#time-threshold).

            Default: CANCEL

        autocomplete : typing.Optional[bool]
            If set to `true`, this payment will be completed when possible. If
            set to `false`, this payment is held in an approved state until either
            explicitly completed (captured) or canceled (voided). For more information, see
            [Delayed capture](https://developer.squareup.com/docs/payments-api/take-payments/card-payments#delayed-capture-of-a-card-payment).

            Default: true

        order_id : typing.Optional[str]
            Associates a previously created order with this payment.

        customer_id : typing.Optional[str]
            The [Customer](entity:Customer) ID of the customer associated with the payment.

            This is required if the `source_id` refers to a card on file created using the Cards API.

        location_id : typing.Optional[str]
            The location ID to associate with the payment. If not specified, the [main location](https://developer.squareup.com/docs/locations-api#about-the-main-location) is
            used.

        team_member_id : typing.Optional[str]
            An optional [TeamMember](entity:TeamMember) ID to associate with
            this payment.

        reference_id : typing.Optional[str]
            A user-defined ID to associate with the payment.

            You can use this field to associate the payment to an entity in an external system
            (for example, you might specify an order ID that is generated by a third-party shopping cart).

        verification_token : typing.Optional[str]
            An identifying token generated by [payments.verifyBuyer()](https://developer.squareup.com/reference/sdks/web/payments/objects/Payments#Payments.verifyBuyer).
            Verification tokens encapsulate customer device information and 3-D Secure
            challenge results to indicate that Square has verified the buyer identity.

            For more information, see [SCA Overview](https://developer.squareup.com/docs/sca-overview).

        accept_partial_authorization : typing.Optional[bool]
            If set to `true` and charging a Square Gift Card, a payment might be returned with
            `amount_money` equal to less than what was requested. For example, a request for $20 when charging
            a Square Gift Card with a balance of $5 results in an APPROVED payment of $5. You might choose
            to prompt the buyer for an additional payment to cover the remainder or cancel the Gift Card
            payment. This field cannot be `true` when `autocomplete = true`.

            For more information, see
            [Partial amount with Square Gift Cards](https://developer.squareup.com/docs/payments-api/take-payments#partial-payment-gift-card).

            Default: false

        buyer_email_address : typing.Optional[str]
            The buyer's email address.

        buyer_phone_number : typing.Optional[str]
            The buyer's phone number.
            Must follow the following format:
            1. A leading + symbol (followed by a country code)
            2. The phone number can contain spaces and the special characters `(` , `)` , `-` , and `.`.
            Alphabetical characters aren't allowed.
            3. The phone number must contain between 9 and 16 digits.

        billing_address : typing.Optional[AddressParams]
            The buyer's billing address.

        shipping_address : typing.Optional[AddressParams]
            The buyer's shipping address.

        note : typing.Optional[str]
            An optional note to be entered by the developer when creating a payment.

        statement_description_identifier : typing.Optional[str]
            Optional additional payment information to include on the customer's card statement
            as part of the statement description. This can be, for example, an invoice number, ticket number,
            or short description that uniquely identifies the purchase.

            Note that the `statement_description_identifier` might get truncated on the statement description
            to fit the required information including the Square identifier (SQ *) and name of the
            seller taking the payment.

        cash_details : typing.Optional[CashPaymentDetailsParams]
            Additional details required when recording a cash payment (`source_id` is CASH).

        external_details : typing.Optional[ExternalPaymentDetailsParams]
            Additional details required when recording an external payment (`source_id` is EXTERNAL).

        customer_details : typing.Optional[CustomerDetailsParams]
            Details about the customer making the payment.

        offline_payment_details : typing.Optional[OfflinePaymentDetailsParams]
            An optional field for specifying the offline payment details. This is intended for
            internal 1st-party callers only.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreatePaymentResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/payments",
            method="POST",
            json={
                "source_id": source_id,
                "idempotency_key": idempotency_key,
                "amount_money": convert_and_respect_annotation_metadata(
                    object_=amount_money, annotation=MoneyParams, direction="write"
                ),
                "tip_money": convert_and_respect_annotation_metadata(
                    object_=tip_money, annotation=MoneyParams, direction="write"
                ),
                "app_fee_money": convert_and_respect_annotation_metadata(
                    object_=app_fee_money, annotation=MoneyParams, direction="write"
                ),
                "delay_duration": delay_duration,
                "delay_action": delay_action,
                "autocomplete": autocomplete,
                "order_id": order_id,
                "customer_id": customer_id,
                "location_id": location_id,
                "team_member_id": team_member_id,
                "reference_id": reference_id,
                "verification_token": verification_token,
                "accept_partial_authorization": accept_partial_authorization,
                "buyer_email_address": buyer_email_address,
                "buyer_phone_number": buyer_phone_number,
                "billing_address": convert_and_respect_annotation_metadata(
                    object_=billing_address, annotation=AddressParams, direction="write"
                ),
                "shipping_address": convert_and_respect_annotation_metadata(
                    object_=shipping_address, annotation=AddressParams, direction="write"
                ),
                "note": note,
                "statement_description_identifier": statement_description_identifier,
                "cash_details": convert_and_respect_annotation_metadata(
                    object_=cash_details, annotation=CashPaymentDetailsParams, direction="write"
                ),
                "external_details": convert_and_respect_annotation_metadata(
                    object_=external_details, annotation=ExternalPaymentDetailsParams, direction="write"
                ),
                "customer_details": convert_and_respect_annotation_metadata(
                    object_=customer_details, annotation=CustomerDetailsParams, direction="write"
                ),
                "offline_payment_details": convert_and_respect_annotation_metadata(
                    object_=offline_payment_details, annotation=OfflinePaymentDetailsParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreatePaymentResponse,
                    construct_type(
                        type_=CreatePaymentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def cancel_by_idempotency_key(
        self, *, idempotency_key: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[CancelPaymentByIdempotencyKeyResponse]:
        """
        Cancels (voids) a payment identified by the idempotency key that is specified in the
        request.

        Use this method when the status of a `CreatePayment` request is unknown (for example, after you send a
        `CreatePayment` request, a network error occurs and you do not get a response). In this case, you can
        direct Square to cancel the payment using this endpoint. In the request, you provide the same
        idempotency key that you provided in your `CreatePayment` request that you want to cancel. After
        canceling the payment, you can submit your `CreatePayment` request again.

        Note that if no payment with the specified idempotency key is found, no action is taken and the endpoint
        returns successfully.

        Parameters
        ----------
        idempotency_key : str
            The `idempotency_key` identifying the payment to be canceled.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CancelPaymentByIdempotencyKeyResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/payments/cancel",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CancelPaymentByIdempotencyKeyResponse,
                    construct_type(
                        type_=CancelPaymentByIdempotencyKeyResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, payment_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetPaymentResponse]:
        """
        Retrieves details for a specific payment.

        Parameters
        ----------
        payment_id : str
            A unique ID for the desired payment.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetPaymentResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/payments/{jsonable_encoder(payment_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetPaymentResponse,
                    construct_type(
                        type_=GetPaymentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        payment_id: str,
        *,
        idempotency_key: str,
        payment: typing.Optional[PaymentParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdatePaymentResponse]:
        """
        Updates a payment with the APPROVED status.
        You can update the `amount_money` and `tip_money` using this endpoint.

        Parameters
        ----------
        payment_id : str
            The ID of the payment to update.

        idempotency_key : str
            A unique string that identifies this `UpdatePayment` request. Keys can be any valid string
            but must be unique for every `UpdatePayment` request.

            For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        payment : typing.Optional[PaymentParams]
            The updated `Payment` object.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdatePaymentResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/payments/{jsonable_encoder(payment_id)}",
            method="PUT",
            json={
                "payment": convert_and_respect_annotation_metadata(
                    object_=payment, annotation=PaymentParams, direction="write"
                ),
                "idempotency_key": idempotency_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdatePaymentResponse,
                    construct_type(
                        type_=UpdatePaymentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def cancel(
        self, payment_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[CancelPaymentResponse]:
        """
        Cancels (voids) a payment. You can use this endpoint to cancel a payment with
        the APPROVED `status`.

        Parameters
        ----------
        payment_id : str
            The ID of the payment to cancel.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CancelPaymentResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/payments/{jsonable_encoder(payment_id)}/cancel",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CancelPaymentResponse,
                    construct_type(
                        type_=CancelPaymentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def complete(
        self,
        payment_id: str,
        *,
        version_token: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CompletePaymentResponse]:
        """
        Completes (captures) a payment.
        By default, payments are set to complete immediately after they are created.

        You can use this endpoint to complete a payment with the APPROVED `status`.

        Parameters
        ----------
        payment_id : str
            The unique ID identifying the payment to be completed.

        version_token : typing.Optional[str]
            Used for optimistic concurrency. This opaque token identifies the current `Payment`
            version that the caller expects. If the server has a different version of the Payment,
            the update fails and a response with a VERSION_MISMATCH error is returned.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CompletePaymentResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/payments/{jsonable_encoder(payment_id)}/complete",
            method="POST",
            json={
                "version_token": version_token,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CompletePaymentResponse,
                    construct_type(
                        type_=CompletePaymentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawPaymentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        source_id: str,
        idempotency_key: str,
        amount_money: typing.Optional[MoneyParams] = OMIT,
        tip_money: typing.Optional[MoneyParams] = OMIT,
        app_fee_money: typing.Optional[MoneyParams] = OMIT,
        delay_duration: typing.Optional[str] = OMIT,
        delay_action: typing.Optional[str] = OMIT,
        autocomplete: typing.Optional[bool] = OMIT,
        order_id: typing.Optional[str] = OMIT,
        customer_id: typing.Optional[str] = OMIT,
        location_id: typing.Optional[str] = OMIT,
        team_member_id: typing.Optional[str] = OMIT,
        reference_id: typing.Optional[str] = OMIT,
        verification_token: typing.Optional[str] = OMIT,
        accept_partial_authorization: typing.Optional[bool] = OMIT,
        buyer_email_address: typing.Optional[str] = OMIT,
        buyer_phone_number: typing.Optional[str] = OMIT,
        billing_address: typing.Optional[AddressParams] = OMIT,
        shipping_address: typing.Optional[AddressParams] = OMIT,
        note: typing.Optional[str] = OMIT,
        statement_description_identifier: typing.Optional[str] = OMIT,
        cash_details: typing.Optional[CashPaymentDetailsParams] = OMIT,
        external_details: typing.Optional[ExternalPaymentDetailsParams] = OMIT,
        customer_details: typing.Optional[CustomerDetailsParams] = OMIT,
        offline_payment_details: typing.Optional[OfflinePaymentDetailsParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreatePaymentResponse]:
        """
        Creates a payment using the provided source. You can use this endpoint
        to charge a card (credit/debit card or
        Square gift card) or record a payment that the seller received outside of Square
        (cash payment from a buyer or a payment that an external entity
        processed on behalf of the seller).

        The endpoint creates a
        `Payment` object and returns it in the response.

        Parameters
        ----------
        source_id : str
            The ID for the source of funds for this payment.
            This could be a payment token generated by the Web Payments SDK for any of its
            [supported methods](https://developer.squareup.com/docs/web-payments/overview#explore-payment-methods),
            including cards, bank transfers, Afterpay or Cash App Pay. If recording a payment
            that the seller received outside of Square, specify either "CASH" or "EXTERNAL".
            For more information, see
            [Take Payments](https://developer.squareup.com/docs/payments-api/take-payments).

        idempotency_key : str
            A unique string that identifies this `CreatePayment` request. Keys can be any valid string
            but must be unique for every `CreatePayment` request.

            Note: The number of allowed characters might be less than the stated maximum, if multi-byte
            characters are used.

            For more information, see [Idempotency](https://developer.squareup.com/docs/working-with-apis/idempotency).

        amount_money : typing.Optional[MoneyParams]
            The amount of money to accept for this payment, not including `tip_money`.

            The amount must be specified in the smallest denomination of the applicable currency
            (for example, US dollar amounts are specified in cents). For more information, see
            [Working with Monetary Amounts](https://developer.squareup.com/docs/build-basics/working-with-monetary-amounts).

            The currency code must match the currency associated with the business
            that is accepting the payment.

        tip_money : typing.Optional[MoneyParams]
            The amount designated as a tip, in addition to `amount_money`.

            The amount must be specified in the smallest denomination of the applicable currency
            (for example, US dollar amounts are specified in cents). For more information, see
            [Working with Monetary Amounts](https://developer.squareup.com/docs/build-basics/working-with-monetary-amounts).

            The currency code must match the currency associated with the business
            that is accepting the payment.

        app_fee_money : typing.Optional[MoneyParams]
            The amount of money that the developer is taking as a fee
            for facilitating the payment on behalf of the seller.

            The amount cannot be more than 90% of the total amount of the payment.

            The amount must be specified in the smallest denomination of the applicable currency
            (for example, US dollar amounts are specified in cents). For more information, see
            [Working with Monetary Amounts](https://developer.squareup.com/docs/build-basics/working-with-monetary-amounts).

            The fee currency code must match the currency associated with the seller
            that is accepting the payment. The application must be from a developer
            account in the same country and using the same currency code as the seller.

            For more information about the application fee scenario, see
            [Take Payments and Collect Fees](https://developer.squareup.com/docs/payments-api/take-payments-and-collect-fees).

            To set this field, `PAYMENTS_WRITE_ADDITIONAL_RECIPIENTS` OAuth permission is required.
            For more information, see [Permissions](https://developer.squareup.com/docs/payments-api/take-payments-and-collect-fees#permissions).

        delay_duration : typing.Optional[str]
            The duration of time after the payment's creation when Square automatically
            either completes or cancels the payment depending on the `delay_action` field value.
            For more information, see
            [Time threshold](https://developer.squareup.com/docs/payments-api/take-payments/card-payments/delayed-capture#time-threshold).

            This parameter should be specified as a time duration, in RFC 3339 format.

            Note: This feature is only supported for card payments. This parameter can only be set for a delayed
            capture payment (`autocomplete=false`).

            Default:

            - Card-present payments: "PT36H" (36 hours) from the creation time.
            - Card-not-present payments: "P7D" (7 days) from the creation time.

        delay_action : typing.Optional[str]
            The action to be applied to the payment when the `delay_duration` has elapsed. The action must be
            CANCEL or COMPLETE. For more information, see
            [Time Threshold](https://developer.squareup.com/docs/payments-api/take-payments/card-payments/delayed-capture#time-threshold).

            Default: CANCEL

        autocomplete : typing.Optional[bool]
            If set to `true`, this payment will be completed when possible. If
            set to `false`, this payment is held in an approved state until either
            explicitly completed (captured) or canceled (voided). For more information, see
            [Delayed capture](https://developer.squareup.com/docs/payments-api/take-payments/card-payments#delayed-capture-of-a-card-payment).

            Default: true

        order_id : typing.Optional[str]
            Associates a previously created order with this payment.

        customer_id : typing.Optional[str]
            The [Customer](entity:Customer) ID of the customer associated with the payment.

            This is required if the `source_id` refers to a card on file created using the Cards API.

        location_id : typing.Optional[str]
            The location ID to associate with the payment. If not specified, the [main location](https://developer.squareup.com/docs/locations-api#about-the-main-location) is
            used.

        team_member_id : typing.Optional[str]
            An optional [TeamMember](entity:TeamMember) ID to associate with
            this payment.

        reference_id : typing.Optional[str]
            A user-defined ID to associate with the payment.

            You can use this field to associate the payment to an entity in an external system
            (for example, you might specify an order ID that is generated by a third-party shopping cart).

        verification_token : typing.Optional[str]
            An identifying token generated by [payments.verifyBuyer()](https://developer.squareup.com/reference/sdks/web/payments/objects/Payments#Payments.verifyBuyer).
            Verification tokens encapsulate customer device information and 3-D Secure
            challenge results to indicate that Square has verified the buyer identity.

            For more information, see [SCA Overview](https://developer.squareup.com/docs/sca-overview).

        accept_partial_authorization : typing.Optional[bool]
            If set to `true` and charging a Square Gift Card, a payment might be returned with
            `amount_money` equal to less than what was requested. For example, a request for $20 when charging
            a Square Gift Card with a balance of $5 results in an APPROVED payment of $5. You might choose
            to prompt the buyer for an additional payment to cover the remainder or cancel the Gift Card
            payment. This field cannot be `true` when `autocomplete = true`.

            For more information, see
            [Partial amount with Square Gift Cards](https://developer.squareup.com/docs/payments-api/take-payments#partial-payment-gift-card).

            Default: false

        buyer_email_address : typing.Optional[str]
            The buyer's email address.

        buyer_phone_number : typing.Optional[str]
            The buyer's phone number.
            Must follow the following format:
            1. A leading + symbol (followed by a country code)
            2. The phone number can contain spaces and the special characters `(` , `)` , `-` , and `.`.
            Alphabetical characters aren't allowed.
            3. The phone number must contain between 9 and 16 digits.

        billing_address : typing.Optional[AddressParams]
            The buyer's billing address.

        shipping_address : typing.Optional[AddressParams]
            The buyer's shipping address.

        note : typing.Optional[str]
            An optional note to be entered by the developer when creating a payment.

        statement_description_identifier : typing.Optional[str]
            Optional additional payment information to include on the customer's card statement
            as part of the statement description. This can be, for example, an invoice number, ticket number,
            or short description that uniquely identifies the purchase.

            Note that the `statement_description_identifier` might get truncated on the statement description
            to fit the required information including the Square identifier (SQ *) and name of the
            seller taking the payment.

        cash_details : typing.Optional[CashPaymentDetailsParams]
            Additional details required when recording a cash payment (`source_id` is CASH).

        external_details : typing.Optional[ExternalPaymentDetailsParams]
            Additional details required when recording an external payment (`source_id` is EXTERNAL).

        customer_details : typing.Optional[CustomerDetailsParams]
            Details about the customer making the payment.

        offline_payment_details : typing.Optional[OfflinePaymentDetailsParams]
            An optional field for specifying the offline payment details. This is intended for
            internal 1st-party callers only.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreatePaymentResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/payments",
            method="POST",
            json={
                "source_id": source_id,
                "idempotency_key": idempotency_key,
                "amount_money": convert_and_respect_annotation_metadata(
                    object_=amount_money, annotation=MoneyParams, direction="write"
                ),
                "tip_money": convert_and_respect_annotation_metadata(
                    object_=tip_money, annotation=MoneyParams, direction="write"
                ),
                "app_fee_money": convert_and_respect_annotation_metadata(
                    object_=app_fee_money, annotation=MoneyParams, direction="write"
                ),
                "delay_duration": delay_duration,
                "delay_action": delay_action,
                "autocomplete": autocomplete,
                "order_id": order_id,
                "customer_id": customer_id,
                "location_id": location_id,
                "team_member_id": team_member_id,
                "reference_id": reference_id,
                "verification_token": verification_token,
                "accept_partial_authorization": accept_partial_authorization,
                "buyer_email_address": buyer_email_address,
                "buyer_phone_number": buyer_phone_number,
                "billing_address": convert_and_respect_annotation_metadata(
                    object_=billing_address, annotation=AddressParams, direction="write"
                ),
                "shipping_address": convert_and_respect_annotation_metadata(
                    object_=shipping_address, annotation=AddressParams, direction="write"
                ),
                "note": note,
                "statement_description_identifier": statement_description_identifier,
                "cash_details": convert_and_respect_annotation_metadata(
                    object_=cash_details, annotation=CashPaymentDetailsParams, direction="write"
                ),
                "external_details": convert_and_respect_annotation_metadata(
                    object_=external_details, annotation=ExternalPaymentDetailsParams, direction="write"
                ),
                "customer_details": convert_and_respect_annotation_metadata(
                    object_=customer_details, annotation=CustomerDetailsParams, direction="write"
                ),
                "offline_payment_details": convert_and_respect_annotation_metadata(
                    object_=offline_payment_details, annotation=OfflinePaymentDetailsParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreatePaymentResponse,
                    construct_type(
                        type_=CreatePaymentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def cancel_by_idempotency_key(
        self, *, idempotency_key: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[CancelPaymentByIdempotencyKeyResponse]:
        """
        Cancels (voids) a payment identified by the idempotency key that is specified in the
        request.

        Use this method when the status of a `CreatePayment` request is unknown (for example, after you send a
        `CreatePayment` request, a network error occurs and you do not get a response). In this case, you can
        direct Square to cancel the payment using this endpoint. In the request, you provide the same
        idempotency key that you provided in your `CreatePayment` request that you want to cancel. After
        canceling the payment, you can submit your `CreatePayment` request again.

        Note that if no payment with the specified idempotency key is found, no action is taken and the endpoint
        returns successfully.

        Parameters
        ----------
        idempotency_key : str
            The `idempotency_key` identifying the payment to be canceled.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CancelPaymentByIdempotencyKeyResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/payments/cancel",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CancelPaymentByIdempotencyKeyResponse,
                    construct_type(
                        type_=CancelPaymentByIdempotencyKeyResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, payment_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetPaymentResponse]:
        """
        Retrieves details for a specific payment.

        Parameters
        ----------
        payment_id : str
            A unique ID for the desired payment.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetPaymentResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/payments/{jsonable_encoder(payment_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetPaymentResponse,
                    construct_type(
                        type_=GetPaymentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        payment_id: str,
        *,
        idempotency_key: str,
        payment: typing.Optional[PaymentParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdatePaymentResponse]:
        """
        Updates a payment with the APPROVED status.
        You can update the `amount_money` and `tip_money` using this endpoint.

        Parameters
        ----------
        payment_id : str
            The ID of the payment to update.

        idempotency_key : str
            A unique string that identifies this `UpdatePayment` request. Keys can be any valid string
            but must be unique for every `UpdatePayment` request.

            For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        payment : typing.Optional[PaymentParams]
            The updated `Payment` object.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdatePaymentResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/payments/{jsonable_encoder(payment_id)}",
            method="PUT",
            json={
                "payment": convert_and_respect_annotation_metadata(
                    object_=payment, annotation=PaymentParams, direction="write"
                ),
                "idempotency_key": idempotency_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdatePaymentResponse,
                    construct_type(
                        type_=UpdatePaymentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def cancel(
        self, payment_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[CancelPaymentResponse]:
        """
        Cancels (voids) a payment. You can use this endpoint to cancel a payment with
        the APPROVED `status`.

        Parameters
        ----------
        payment_id : str
            The ID of the payment to cancel.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CancelPaymentResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/payments/{jsonable_encoder(payment_id)}/cancel",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CancelPaymentResponse,
                    construct_type(
                        type_=CancelPaymentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def complete(
        self,
        payment_id: str,
        *,
        version_token: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CompletePaymentResponse]:
        """
        Completes (captures) a payment.
        By default, payments are set to complete immediately after they are created.

        You can use this endpoint to complete a payment with the APPROVED `status`.

        Parameters
        ----------
        payment_id : str
            The unique ID identifying the payment to be completed.

        version_token : typing.Optional[str]
            Used for optimistic concurrency. This opaque token identifies the current `Payment`
            version that the caller expects. If the server has a different version of the Payment,
            the update fails and a response with a VERSION_MISMATCH error is returned.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CompletePaymentResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/payments/{jsonable_encoder(payment_id)}/complete",
            method="POST",
            json={
                "version_token": version_token,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CompletePaymentResponse,
                    construct_type(
                        type_=CompletePaymentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
import typing
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ReceiptOptions(UncheckedBaseModel):
    """
    Describes receipt action fields.
    """

    payment_id: str = pydantic.Field()
    """
    The reference to the Square payment ID for the receipt.
    """

    print_only: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Instructs the device to print the receipt without displaying the receipt selection screen.
    Requires `printer_enabled` set to true.
    Defaults to false.
    """

    is_duplicate: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Identify the receipt as a reprint rather than an original receipt.
    Defaults to false.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class ClearpayDetailsParams(typing_extensions.TypedDict):
    """
    Additional details about Clearpay payments.
    """

    email_address: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Email address on the buyer's Clearpay account.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

GiftCardActivityDeactivateReason = typing.Union[
    typing.Literal["SUSPICIOUS_ACTIVITY", "UNKNOWN_REASON", "CHARGEBACK_DEACTIVATE"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .catalog_object import CatalogObjectParams
from .catalog_id_mapping import CatalogIdMappingParams


class UpsertCatalogObjectResponseParams(typing_extensions.TypedDict):
    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    catalog_object: typing_extensions.NotRequired[CatalogObjectParams]
    """
    The successfully created or updated CatalogObject.
    """

    id_mappings: typing_extensions.NotRequired[typing.Sequence[CatalogIdMappingParams]]
    """
    The mapping between client and server IDs for this upsert.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .search_subscriptions_filter import SearchSubscriptionsFilterParams


class SearchSubscriptionsQueryParams(typing_extensions.TypedDict):
    """
    Represents a query, consisting of specified query expressions, used to search for subscriptions.
    """

    filter: typing_extensions.NotRequired[SearchSubscriptionsFilterParams]
    """
    A list of query expressions.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .order_source import OrderSourceParams
from .order_line_item import OrderLineItemParams
from .order_line_item_tax import OrderLineItemTaxParams
from .order_line_item_discount import OrderLineItemDiscountParams
from .order_service_charge import OrderServiceChargeParams
from .fulfillment import FulfillmentParams
from .order_return import OrderReturnParams
from .order_money_amounts import OrderMoneyAmountsParams
from .order_rounding_adjustment import OrderRoundingAdjustmentParams
from .tender import TenderParams
from .refund import RefundParams
from ..types.order_state import OrderState
from .money import MoneyParams
from .order_pricing_options import OrderPricingOptionsParams
from .order_reward import OrderRewardParams


class OrderParams(typing_extensions.TypedDict):
    """
    Contains all information related to a single order to process with Square,
    including line items that specify the products to purchase. `Order` objects also
    include information about any associated tenders, refunds, and returns.

    All Connect V2 Transactions have all been converted to Orders including all associated
    itemization data.
    """

    id: typing_extensions.NotRequired[str]
    """
    The order's unique ID.
    """

    location_id: str
    """
    The ID of the seller location that this order is associated with.
    """

    reference_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A client-specified ID to associate an entity in another system
    with this order.
    """

    source: typing_extensions.NotRequired[OrderSourceParams]
    """
    The origination details of the order.
    """

    customer_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the [customer](entity:Customer) associated with the order.
    
    You should specify a `customer_id` on the order (or the payment) to ensure that transactions
    are reliably linked to customers. Omitting this field might result in the creation of new
    [instant profiles](https://developer.squareup.com/docs/customers-api/what-it-does#instant-profiles).
    """

    line_items: typing_extensions.NotRequired[typing.Optional[typing.Sequence[OrderLineItemParams]]]
    """
    The line items included in the order.
    """

    taxes: typing_extensions.NotRequired[typing.Optional[typing.Sequence[OrderLineItemTaxParams]]]
    """
    The list of all taxes associated with the order.
    
    Taxes can be scoped to either `ORDER` or `LINE_ITEM`. For taxes with `LINE_ITEM` scope, an
    `OrderLineItemAppliedTax` must be added to each line item that the tax applies to. For taxes
    with `ORDER` scope, the server generates an `OrderLineItemAppliedTax` for every line item.
    
    On reads, each tax in the list includes the total amount of that tax applied to the order.
    
    __IMPORTANT__: If `LINE_ITEM` scope is set on any taxes in this field, using the deprecated
    `line_items.taxes` field results in an error. Use `line_items.applied_taxes`
    instead.
    """

    discounts: typing_extensions.NotRequired[typing.Optional[typing.Sequence[OrderLineItemDiscountParams]]]
    """
    The list of all discounts associated with the order.
    
    Discounts can be scoped to either `ORDER` or `LINE_ITEM`. For discounts scoped to `LINE_ITEM`,
    an `OrderLineItemAppliedDiscount` must be added to each line item that the discount applies to.
    For discounts with `ORDER` scope, the server generates an `OrderLineItemAppliedDiscount`
    for every line item.
    
    __IMPORTANT__: If `LINE_ITEM` scope is set on any discounts in this field, using the deprecated
    `line_items.discounts` field results in an error. Use `line_items.applied_discounts`
    instead.
    """

    service_charges: typing_extensions.NotRequired[typing.Optional[typing.Sequence[OrderServiceChargeParams]]]
    """
    A list of service charges applied to the order.
    """

    fulfillments: typing_extensions.NotRequired[typing.Optional[typing.Sequence[FulfillmentParams]]]
    """
    Details about order fulfillment.
    
    Orders can only be created with at most one fulfillment. However, orders returned
    by the API might contain multiple fulfillments.
    """

    returns: typing_extensions.NotRequired[typing.Sequence[OrderReturnParams]]
    """
    A collection of items from sale orders being returned in this one. Normally part of an
    itemized return or exchange. There is exactly one `Return` object per sale `Order` being
    referenced.
    """

    return_amounts: typing_extensions.NotRequired[OrderMoneyAmountsParams]
    """
    The rollup of the returned money amounts.
    """

    net_amounts: typing_extensions.NotRequired[OrderMoneyAmountsParams]
    """
    The net money amounts (sale money - return money).
    """

    rounding_adjustment: typing_extensions.NotRequired[OrderRoundingAdjustmentParams]
    """
    A positive rounding adjustment to the total of the order. This adjustment is commonly
    used to apply cash rounding when the minimum unit of account is smaller than the lowest physical
    denomination of the currency.
    """

    tenders: typing_extensions.NotRequired[typing.Sequence[TenderParams]]
    """
    The tenders that were used to pay for the order.
    """

    refunds: typing_extensions.NotRequired[typing.Sequence[RefundParams]]
    """
    The refunds that are part of this order.
    """

    metadata: typing_extensions.NotRequired[typing.Optional[typing.Dict[str, typing.Optional[str]]]]
    """
    Application-defined data attached to this order. Metadata fields are intended
    to store descriptive references or associations with an entity in another system or store brief
    information about the object. Square does not process this field; it only stores and returns it
    in relevant API calls. Do not use metadata to store any sensitive information (such as personally
    identifiable information or card details).
    
    Keys written by applications must be 60 characters or less and must be in the character set
    `[a-zA-Z0-9_-]`. Entries can also include metadata generated by Square. These keys are prefixed
    with a namespace, separated from the key with a ':' character.
    
    Values have a maximum length of 255 characters.
    
    An application can have up to 10 entries per metadata field.
    
    Entries written by applications are private and can only be read or modified by the same
    application.
    
    For more information, see  [Metadata](https://developer.squareup.com/docs/build-basics/metadata).
    """

    created_at: typing_extensions.NotRequired[str]
    """
    The timestamp for when the order was created, at server side, in RFC 3339 format (for example, "2016-09-04T23:59:33.123Z").
    """

    updated_at: typing_extensions.NotRequired[str]
    """
    The timestamp for when the order was last updated, at server side, in RFC 3339 format (for example, "2016-09-04T23:59:33.123Z").
    """

    closed_at: typing_extensions.NotRequired[str]
    """
    The timestamp for when the order reached a terminal [state](entity:OrderState), in RFC 3339 format (for example "2016-09-04T23:59:33.123Z").
    """

    state: typing_extensions.NotRequired[OrderState]
    """
    The current state of the order.
    See [OrderState](#type-orderstate) for possible values
    """

    version: typing_extensions.NotRequired[int]
    """
    The version number, which is incremented each time an update is committed to the order.
    Orders not created through the API do not include a version number and
    therefore cannot be updated.
    
    [Read more about working with versions](https://developer.squareup.com/docs/orders-api/manage-orders/update-orders).
    """

    total_money: typing_extensions.NotRequired[MoneyParams]
    """
    The total amount of money to collect for the order.
    """

    total_tax_money: typing_extensions.NotRequired[MoneyParams]
    """
    The total amount of tax money to collect for the order.
    """

    total_discount_money: typing_extensions.NotRequired[MoneyParams]
    """
    The total amount of discount money to collect for the order.
    """

    total_tip_money: typing_extensions.NotRequired[MoneyParams]
    """
    The total amount of tip money to collect for the order.
    """

    total_service_charge_money: typing_extensions.NotRequired[MoneyParams]
    """
    The total amount of money collected in service charges for the order.
    
    Note: `total_service_charge_money` is the sum of `applied_money` fields for each individual
    service charge. Therefore, `total_service_charge_money` only includes inclusive tax amounts,
    not additive tax amounts.
    """

    ticket_name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A short-term identifier for the order (such as a customer first name,
    table number, or auto-generated order number that resets daily).
    """

    pricing_options: typing_extensions.NotRequired[OrderPricingOptionsParams]
    """
    Pricing options for an order. The options affect how the order's price is calculated.
    They can be used, for example, to apply automatic price adjustments that are based on
    preconfigured [pricing rules](entity:CatalogPricingRule).
    """

    rewards: typing_extensions.NotRequired[typing.Sequence[OrderRewardParams]]
    """
    A set-like list of Rewards that have been added to the Order.
    """

    net_amount_due_money: typing_extensions.NotRequired[MoneyParams]
    """
    The net amount of money due on the order.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class LoyaltyProgramAccrualRuleCategoryData(UncheckedBaseModel):
    """
    Represents additional data for rules with the `CATEGORY` accrual type.
    """

    category_id: str = pydantic.Field()
    """
    The ID of the `CATEGORY` [catalog object](entity:CatalogObject) that buyers can purchase to earn
    points.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
from .catalog_category import CatalogCategory
from .catalog_item import CatalogItem
from .catalog_item_option import CatalogItemOption
from .catalog_modifier_list import CatalogModifierList
from .catalog_object_category import CatalogObjectCategory
from .catalog_object_item import CatalogObjectItem
from .catalog_object_item_option import CatalogObjectItemOption
from .catalog_object_modifier_list import CatalogObjectModifierList
from .catalog_object_subscription_plan import CatalogObjectSubscriptionPlan
from .catalog_subscription_plan import CatalogSubscriptionPlan
import pydantic
import typing
from .catalog_object import CatalogObject
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CreateCatalogImageRequest(UncheckedBaseModel):
    idempotency_key: str = pydantic.Field()
    """
    A unique string that identifies this CreateCatalogImage request.
    Keys can be any valid string but must be unique for every CreateCatalogImage request.
    
    See [Idempotency keys](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) for more information.
    """

    object_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique ID of the `CatalogObject` to attach this `CatalogImage` object to. Leave this
    field empty to create unattached images, for example if you are building an integration
    where an image can be attached to catalog items at a later time.
    """

    image: CatalogObject = pydantic.Field()
    """
    The new `CatalogObject` of the `IMAGE` type, namely, a `CatalogImage` object, to encapsulate the specified image file.
    """

    is_primary: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If this is set to `true`, the image created will be the primary, or first image of the object referenced by `object_id`.
    If the `CatalogObject` already has a primary `CatalogImage`, setting this field to `true` will replace the primary image.
    If this is set to `false` and you use the Square API version 2021-12-15 or later, the image id will be appended to the list of `image_ids` on the object.
    
    With Square API version 2021-12-15 or later, the default value is `false`. Otherwise, the effective default value is `true`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions


class SignatureImageParams(typing_extensions.TypedDict):
    image_type: typing_extensions.NotRequired[str]
    """
    The mime/type of the image data.
    Use `image/png;base64` for png.
    """

    data: typing_extensions.NotRequired[str]
    """
    The base64 representation of the image.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class VendorContactParams(typing_extensions.TypedDict):
    """
    Represents a contact of a [Vendor](entity:Vendor).
    """

    id: typing_extensions.NotRequired[str]
    """
    A unique Square-generated ID for the [VendorContact](entity:VendorContact).
    This field is required when attempting to update a [VendorContact](entity:VendorContact).
    """

    name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The name of the [VendorContact](entity:VendorContact).
    This field is required when attempting to create a [Vendor](entity:Vendor).
    """

    email_address: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The email address of the [VendorContact](entity:VendorContact).
    """

    phone_number: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The phone number of the [VendorContact](entity:VendorContact).
    """

    removed: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    The state of the [VendorContact](entity:VendorContact).
    """

    ordinal: int
    """
    The ordinal of the [VendorContact](entity:VendorContact).
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .job import Job
import pydantic
from .error import Error
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ListJobsResponse(UncheckedBaseModel):
    """
    Represents a [ListJobs](api-endpoint:Team-ListJobs) response. Either `jobs` or `errors`
    is present in the response. If additional results are available, the `cursor` field is also present.
    """

    jobs: typing.Optional[typing.List[Job]] = pydantic.Field(default=None)
    """
    The retrieved jobs. A single paged response contains up to 100 jobs.
    """

    cursor: typing.Optional[str] = pydantic.Field(default=None)
    """
    An opaque cursor used to retrieve the next page of results. This field is present only
    if the request succeeded and additional results are available. For more information, see
    [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    The errors that occurred during the request.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .team_member import TeamMember
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CreateTeamMemberRequest(UncheckedBaseModel):
    """
    Represents a create request for a `TeamMember` object.
    """

    idempotency_key: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique string that identifies this `CreateTeamMember` request.
    Keys can be any valid string, but must be unique for every request.
    For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).
    
    The minimum length is 1 and the maximum length is 45.
    """

    team_member: typing.Optional[TeamMember] = pydantic.Field(default=None)
    """
    **Required** The data used to create the `TeamMember` object. If you include `wage_setting`, you must provide
    `job_id` for each job assignment. To get job IDs, call [ListJobs](api-endpoint:Team-ListJobs).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .customer_segment import CustomerSegmentParams


class ListCustomerSegmentsResponseParams(typing_extensions.TypedDict):
    """
    Defines the fields that are included in the response body for requests to the `ListCustomerSegments` endpoint.

    Either `errors` or `segments` is present in a given response (never both).
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    segments: typing_extensions.NotRequired[typing.Sequence[CustomerSegmentParams]]
    """
    The list of customer segments belonging to the associated Square account.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    A pagination cursor to be used in subsequent calls to `ListCustomerSegments`
    to retrieve the next set of query results. The cursor is only present if the request succeeded and
    additional results are available.
    
    For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.exclude_strategy import ExcludeStrategy
from .money import MoneyParams


class CatalogPricingRuleParams(typing_extensions.TypedDict):
    """
    Defines how discounts are automatically applied to a set of items that match the pricing rule
    during the active time period.
    """

    name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    User-defined name for the pricing rule. For example, "Buy one get one
    free" or "10% off".
    """

    time_period_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    A list of unique IDs for the catalog time periods when
    this pricing rule is in effect. If left unset, the pricing rule is always
    in effect.
    """

    discount_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Unique ID for the `CatalogDiscount` to take off
    the price of all matched items.
    """

    match_products_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Unique ID for the `CatalogProductSet` that will be matched by this rule. A match rule
    matches within the entire cart, and can match multiple times. This field will always be set.
    """

    apply_products_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    __Deprecated__: Please use the `exclude_products_id` field to apply
    an exclude set instead. Exclude sets allow better control over quantity
    ranges and offer more flexibility for which matched items receive a discount.
    
    `CatalogProductSet` to apply the pricing to.
    An apply rule matches within the subset of the cart that fits the match rules (the match set).
    An apply rule can only match once in the match set.
    If not supplied, the pricing will be applied to all products in the match set.
    Other products retain their base price, or a price generated by other rules.
    """

    exclude_products_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    `CatalogProductSet` to exclude from the pricing rule.
    An exclude rule matches within the subset of the cart that fits the match rules (the match set).
    An exclude rule can only match once in the match set.
    If not supplied, the pricing will be applied to all products in the match set.
    Other products retain their base price, or a price generated by other rules.
    """

    valid_from_date: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Represents the date the Pricing Rule is valid from. Represented in RFC 3339 full-date format (YYYY-MM-DD).
    """

    valid_from_local_time: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Represents the local time the pricing rule should be valid from. Represented in RFC 3339 partial-time format
    (HH:MM:SS). Partial seconds will be truncated.
    """

    valid_until_date: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Represents the date the Pricing Rule is valid until. Represented in RFC 3339 full-date format (YYYY-MM-DD).
    """

    valid_until_local_time: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Represents the local time the pricing rule should be valid until. Represented in RFC 3339 partial-time format
    (HH:MM:SS). Partial seconds will be truncated.
    """

    exclude_strategy: typing_extensions.NotRequired[ExcludeStrategy]
    """
    If an `exclude_products_id` was given, controls which subset of matched
    products is excluded from any discounts.
    
    Default value: `LEAST_EXPENSIVE`
    See [ExcludeStrategy](#type-excludestrategy) for possible values
    """

    minimum_order_subtotal_money: typing_extensions.NotRequired[MoneyParams]
    """
    The minimum order subtotal (before discounts or taxes are applied)
    that must be met before this rule may be applied.
    """

    customer_group_ids_any: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    A list of IDs of customer groups, the members of which are eligible for discounts specified in this pricing rule.
    Notice that a group ID is generated by the Customers API.
    If this field is not set, the specified discount applies to matched products sold to anyone whether the buyer
    has a customer profile created or not. If this `customer_group_ids_any` field is set, the specified discount
    applies only to matched products sold to customers belonging to the specified customer groups.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .checkout_merchant_settings_payment_methods_payment_method import (
    CheckoutMerchantSettingsPaymentMethodsPaymentMethod,
)
from .checkout_merchant_settings_payment_methods_afterpay_clearpay import (
    CheckoutMerchantSettingsPaymentMethodsAfterpayClearpay,
)
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class CheckoutMerchantSettingsPaymentMethods(UncheckedBaseModel):
    apple_pay: typing.Optional[CheckoutMerchantSettingsPaymentMethodsPaymentMethod] = None
    google_pay: typing.Optional[CheckoutMerchantSettingsPaymentMethodsPaymentMethod] = None
    cash_app: typing.Optional[CheckoutMerchantSettingsPaymentMethodsPaymentMethod] = None
    afterpay_clearpay: typing.Optional[CheckoutMerchantSettingsPaymentMethodsAfterpayClearpay] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
